@@ -17,7 +17,10 @@ import subprocess
 import shutil
 import argparse
 from .. import mlog
+from ..mesonlib import has_path_sep
 from . import destdir_join
+from .gettext import read_linguas
+import typing as T
 
 parser = argparse.ArgumentParser()
 parser.add_argument('command')
@@ -29,19 +32,19 @@ parser.add_argument('--media', dest='media', default='')
 parser.add_argument('--langs', dest='langs', default='')
 parser.add_argument('--symlinks', type=bool, dest='symlinks', default=False)
 
-def build_pot(srcdir, project_id, sources):
+def build_pot(srcdir: str, project_id: str, sources: T.List[str]) -> None:
     # Must be relative paths
     sources = [os.path.join('C', source) for source in sources]
     outfile = os.path.join(srcdir, project_id + '.pot')
     subprocess.call(['itstool', '-o', outfile] + sources)
 
-def update_po(srcdir, project_id, langs):
+def update_po(srcdir: str, project_id: str, langs: T.List[str]) -> None:
     potfile = os.path.join(srcdir, project_id + '.pot')
     for lang in langs:
         pofile = os.path.join(srcdir, lang, lang + '.po')
         subprocess.call(['msgmerge', '-q', '-o', pofile, pofile, potfile])
 
-def build_translations(srcdir, blddir, langs):
+def build_translations(srcdir: str, blddir: str, langs: T.List[str]) -> None:
     for lang in langs:
         outdir = os.path.join(blddir, lang)
         os.makedirs(outdir, exist_ok=True)
@@ -50,14 +53,14 @@ def build_translations(srcdir, blddir, langs):
             '-o', os.path.join(outdir, lang + '.gmo')
         ])
 
-def merge_translations(blddir, sources, langs):
+def merge_translations(blddir: str, sources: T.List[str], langs: T.List[str]) -> None:
     for lang in langs:
         subprocess.call([
             'itstool', '-m', os.path.join(blddir, lang, lang + '.gmo'),
             '-o', os.path.join(blddir, lang)
         ] + sources)
 
-def install_help(srcdir, blddir, sources, media, langs, install_dir, destdir, project_id, symlinks):
+def install_help(srcdir: str, blddir: str, sources: T.List[str], media: T.List[str], langs: T.List[str], install_dir: str, destdir: str, project_id: str, symlinks: bool) -> None:
     c_install_dir = os.path.join(install_dir, 'C', project_id)
     for lang in langs + ['C']:
         indir = destdir_join(destdir, os.path.join(install_dir, lang, project_id))
@@ -66,28 +69,40 @@ def install_help(srcdir, blddir, sources, media, langs, install_dir, destdir, pr
             infile = os.path.join(srcdir if lang == 'C' else blddir, lang, source)
             outfile = os.path.join(indir, source)
             mlog.log('Installing %s to %s' % (infile, outfile))
-            shutil.copyfile(infile, outfile)
-            shutil.copystat(infile, outfile)
+            shutil.copy2(infile, outfile)
         for m in media:
             infile = os.path.join(srcdir, lang, m)
             outfile = os.path.join(indir, m)
+            c_infile = os.path.join(srcdir, 'C', m)
             if not os.path.exists(infile):
-                if lang == 'C':
+                if not os.path.exists(c_infile):
                     mlog.warning('Media file "%s" did not exist in C directory' % m)
+                    continue
                 elif symlinks:
                     srcfile = os.path.join(c_install_dir, m)
                     mlog.log('Symlinking %s to %s.' % (outfile, srcfile))
-                    if '/' in m or '\\' in m:
+                    if has_path_sep(m):
                         os.makedirs(os.path.dirname(outfile), exist_ok=True)
-                    os.symlink(srcfile, outfile)
-                continue
+                    try:
+                        try:
+                            os.symlink(srcfile, outfile)
+                        except FileExistsError:
+                            os.remove(outfile)
+                            os.symlink(srcfile, outfile)
+                        continue
+                    except (NotImplementedError, OSError):
+                        mlog.warning('Symlinking not supported, falling back to copying')
+                        infile = c_infile
+                else:
+                    # Lang doesn't have media file so copy it over 'C' one
+                    infile = c_infile
             mlog.log('Installing %s to %s' % (infile, outfile))
-            if '/' in m or '\\' in m:
+            if has_path_sep(m):
                 os.makedirs(os.path.dirname(outfile), exist_ok=True)
             shutil.copyfile(infile, outfile)
             shutil.copystat(infile, outfile)
 
-def run(args):
+def run(args: T.List[str]) -> int:
     options = parser.parse_args(args)
     langs = options.langs.split('@@') if options.langs else []
     media = options.media.split('@@') if options.media else []
@@ -97,6 +112,9 @@ def run(args):
     build_subdir = os.path.join(os.environ['MESON_BUILD_ROOT'], options.subdir)
     abs_sources = [os.path.join(src_subdir, 'C', source) for source in sources]
 
+    if not langs:
+        langs = read_linguas(src_subdir)
+
     if options.command == 'pot':
         build_pot(src_subdir, options.project_id, sources)
     elif options.command == 'update-po':
@@ -112,3 +130,4 @@ def run(args):
             merge_translations(build_subdir, abs_sources, langs)
         install_help(src_subdir, build_subdir, sources, media, langs, install_dir,
                      destdir, options.project_id, options.symlinks)
+    return 0
